@@ -19,13 +19,14 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.          #
 # ------------------------------------------------------------------------------------------------ #
 
-__version__ = '0.1.4'
+__version__ = '0.1.6'
 
 
 # expose specific classes and functions
 from ._core.v import V
 from ._core.q import Q
 from ._core.policy import Policy
+from ._core.worker import Worker
 from ._core.reward_function import RewardFunction
 from ._core.transition_model import TransitionModel
 from ._core.stochastic_v import StochasticV
@@ -57,6 +58,7 @@ __all__ = (
     'V',
     'Q',
     'Policy',
+    'Worker',
     'RewardFunction',
     'TransitionModel',
     'StochasticV',
