@@ -20,14 +20,14 @@
 # ------------------------------------------------------------------------------------------------ #
 
 from ._add_orig_to_info import AddOrigToInfoDictMixin
+from ._copy import CopyMixin
 from ._logger import LoggerMixin
 from ._random_state import RandomStateMixin
-from ._serialization import SerializationMixin
 
 
 __all__ = (
     'AddOrigToInfoDictMixin',
+    'CopyMixin',
     'LoggerMixin',
     'RandomStateMixin',
-    'SerializationMixin',
 )
