@@ -28,7 +28,7 @@ import optax
 import chex
 
 from .._base.mixins import RandomStateMixin
-from ..utils import get_grads_diagnostics, is_policy, is_stochastic, is_qfunction, is_vfunction
+from ..utils import get_grads_diagnostics, is_policy, is_stochastic, is_qfunction, is_vfunction, jit
 from ..value_losses import huber
 from ..regularizers import Regularizer
 
@@ -60,7 +60,7 @@ class BaseTDLearning(ABC, RandomStateMixin):
             new_params = optax.apply_updates(params, updates)
             return new_opt_state, new_params
 
-        self._apply_grads_func = jax.jit(apply_grads_func, static_argnums=0)
+        self._apply_grads_func = jit(apply_grads_func, static_argnums=0)
 
     @abstractmethod
     def target_func(self, target_params, target_state, rng, transition_batch):
@@ -76,7 +76,7 @@ class BaseTDLearning(ABC, RandomStateMixin):
     def target_function_state(self):
         pass
 
-    def update(self, transition_batch):
+    def update(self, transition_batch, return_td_error=False):
         r"""
 
         Update the model parameters (weights) of the underlying function approximator.
@@ -87,18 +87,27 @@ class BaseTDLearning(ABC, RandomStateMixin):
 
             A batch of transitions.
 
+        return_td_error : bool, optional
+
+            Whether to return the TD-errors.
+
         Returns
         -------
         metrics : dict of scalar ndarrays
 
             The structure of the metrics dict is ``{name: score}``.
 
+        td_error : ndarray, optional
+
+            The non-aggregated TD-errors, :code:`shape == (batch_size,)`. This is only returned if
+            we set :code:`return_td_error=True`.
+
         """
-        grads, function_state, metrics = self.grads_and_metrics(transition_batch)
+        grads, function_state, metrics, td_error = self.grads_and_metrics(transition_batch)
         if any(jnp.any(jnp.isnan(g)) for g in jax.tree_leaves(grads)):
             raise RuntimeError(f"found nan's in grads: {grads}")
         self.update_from_grads(grads, function_state)
-        return metrics
+        return (metrics, td_error) if return_td_error else metrics
 
     def update_from_grads(self, grads, function_state):
         r"""
@@ -151,6 +160,10 @@ class BaseTDLearning(ABC, RandomStateMixin):
 
             The structure of the metrics dict is ``{name: score}``.
 
+        td_error : ndarray
+
+            The non-aggregated TD-errors, :code:`shape == (batch_size,)`.
+
         """
         return self._grads_and_metrics_func(
             self._f.params, self.target_params, self._f.function_state, self.target_function_state,
@@ -285,10 +298,11 @@ class BaseTDLearningV(BaseTDLearning):
             chex.assert_equal_shape([G, V, V_targ, W])
             chex.assert_rank([G, V, V_targ, W], 1)
             dLoss_dV = jax.grad(self.loss_function, argnums=1)
-            td_error = -V.shape[0] * dLoss_dV(G, V, W)  # e.g. (G - V) if loss function is MSE
+            td_error = -V.shape[0] * dLoss_dV(G, V)  # e.g. (G - V) if loss function is MSE
+            chex.assert_equal_shape([td_error, W])
             metrics = {
                 f'{self.__class__.__name__}/loss': loss,
-                f'{self.__class__.__name__}/td_error': jnp.mean(td_error),
+                f'{self.__class__.__name__}/td_error': jnp.mean(W * td_error),
                 f'{self.__class__.__name__}/td_error_targ': jnp.mean(-dLoss_dV(V, V_targ, W)),
             }
             return loss, (td_error, state_new, metrics)
@@ -303,15 +317,15 @@ class BaseTDLearningV(BaseTDLearning):
             # add some diagnostics about the gradients
             metrics.update(get_grads_diagnostics(grads, f'{self.__class__.__name__}/grads_'))
 
-            return grads, state_new, metrics
+            return grads, state_new, metrics, td_error
 
         def td_error_func(params, target_params, state, target_state, rng, transition_batch):
             loss, (td_error, state_new, metrics) =\
                 loss_func(params, target_params, state, target_state, rng, transition_batch)
             return td_error
 
-        self._grads_and_metrics_func = jax.jit(grads_and_metrics_func)
-        self._td_error_func = jax.jit(td_error_func)
+        self._grads_and_metrics_func = jit(grads_and_metrics_func)
+        self._td_error_func = jit(td_error_func)
 
     @property
     def v(self):
@@ -431,10 +445,11 @@ class BaseTDLearningQ(BaseTDLearning):
             chex.assert_equal_shape([G, Q, Q_targ, W])
             chex.assert_rank([G, Q, Q_targ, W], 1)
             dLoss_dQ = jax.grad(self.loss_function, argnums=1)
-            td_error = -Q.shape[0] * dLoss_dQ(G, Q, W)  # e.g. (G - Q) if loss function is MSE
+            td_error = -Q.shape[0] * dLoss_dQ(G, Q)  # e.g. (G - Q) if loss function is MSE
+            chex.assert_equal_shape([td_error, W])
             metrics = {
                 f'{self.__class__.__name__}/loss': loss,
-                f'{self.__class__.__name__}/td_error': jnp.mean(td_error),
+                f'{self.__class__.__name__}/td_error': jnp.mean(W * td_error),
                 f'{self.__class__.__name__}/td_error_targ': jnp.mean(-dLoss_dQ(Q, Q_targ, W)),
             }
             return loss, (td_error, state_new, metrics)
@@ -449,15 +464,15 @@ class BaseTDLearningQ(BaseTDLearning):
             # add some diagnostics about the gradients
             metrics.update(get_grads_diagnostics(grads, f'{self.__class__.__name__}/grads_'))
 
-            return grads, state_new, metrics
+            return grads, state_new, metrics, td_error
 
         def td_error_func(params, target_params, state, target_state, rng, transition_batch):
             loss, (td_error, state_new, metrics) =\
                 loss_func(params, target_params, state, target_state, rng, transition_batch)
             return td_error
 
-        self._grads_and_metrics_func = jax.jit(grads_and_metrics_func)
-        self._td_error_func = jax.jit(td_error_func)
+        self._grads_and_metrics_func = jit(grads_and_metrics_func)
+        self._td_error_func = jit(td_error_func)
 
     @property
     def q(self):
