@@ -19,7 +19,7 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.          #
 # ------------------------------------------------------------------------------------------------ #
 
-from typing import TypeVar, Union, Sequence
+from typing import TypeVar, Union, Sequence, Callable, Tuple
 
 
 __all__ = (
@@ -34,3 +34,7 @@ Batch = Sequence                           # annotate batched quantities
 Observation = TypeVar('Observation')       # a state observation
 Action = TypeVar('Action')                 # an action
 SpaceElement = Union[Observation, Action]  # element of a gym-style space
+LogPropensity = TypeVar('LogPropensity')   # an action
+
+
+Policy = Callable[[Observation, bool], Union[Action, Tuple[Action, LogPropensity]]]
