@@ -24,15 +24,23 @@ import time
 import logging
 from importlib import reload, import_module
 from types import ModuleType
+from typing import NamedTuple
 
 import jax.numpy as jnp
 import numpy as onp
+import pandas as pd
+import lz4.frame
+import cloudpickle as pickle
 from PIL import Image
 
 
 __all__ = (
     'docstring',
     'enable_logging',
+    'dump',
+    'dumps',
+    'load',
+    'loads',
     'generate_gif',
     'get_env_attr',
     'getattr_safe',
@@ -44,6 +52,7 @@ __all__ = (
     'is_transition_model',
     'is_vfunction',
     'pretty_repr',
+    'pretty_print',
     'reload_recursive',
     'render_episode',
 )
@@ -132,17 +141,154 @@ def enable_logging(name=None, level=logging.INFO, output_filepath=None, output_l
 
     """
     if name is None:
-        fmt = '[%(threadName)s|%(name)s|%(levelname)s] %(message)s'
+        fmt = '[%(name)s|%(levelname)s] %(message)s'
     else:
-        fmt = f'[{name}|%(threadName)s|%(name)s|%(levelname)s] %(message)s'
+        fmt = f'[{name}|%(name)s|%(levelname)s] %(message)s'
     logging.basicConfig(level=level, format=fmt)
     if output_filepath is not None:
-        os.makedirs(os.path.dirname(output_filepath), exist_ok=True)
+        os.makedirs(os.path.dirname(output_filepath) or '.', exist_ok=True)
         fh = logging.FileHandler(output_filepath)
         fh.setLevel(level if output_level is None else output_level)
         logging.getLogger('').addHandler(fh)
 
 
+def dump(obj, filepath):
+    r"""
+
+    Save an object to disk.
+
+    Parameters
+    ----------
+    obj : object
+
+        Any python object.
+
+    filepath : str
+
+        Where to store the instance.
+
+    Warning
+    -------
+
+    References between objects are only preserved if they are stored as part of a single object, for
+    example:
+
+    .. code:: python
+
+        # b has a reference to a
+        a = [13]
+        b = {'a': a}
+
+        # references preserved
+        dump((a, b), 'ab.pkl.lz4')
+        a_new, b_new = load('ab.pkl.lz4')
+        b_new['a'].append(7)
+        print(b_new)  # {'a': [13, 7]}
+        print(a_new)  # [13, 7]         <-- updated
+
+        # references not preserved
+        dump(a, 'a.pkl.lz4')
+        dump(b, 'b.pkl.lz4')
+        a_new = load('a.pkl.lz4')
+        b_new = load('b.pkl.lz4')
+        b_new['a'].append(7)
+        print(b_new)  # {'a': [13, 7]}
+        print(a_new)  # [13]            <-- not updated!!
+
+    Therefore, the safest way to create checkpoints is to store the entire state as a single object
+    like a dict or a tuple.
+
+    """
+    dirpath = os.path.dirname(filepath)
+    if dirpath:
+        os.makedirs(dirpath, exist_ok=True)
+    with lz4.frame.open(filepath, 'wb') as f:
+        f.write(pickle.dumps(obj))
+
+
+def dumps(obj):
+    r"""
+
+    Serialize an object to an lz4-compressed pickle byte-string.
+
+    Parameters
+    ----------
+    obj : object
+
+        Any python object.
+
+    Returns
+    -------
+    s : bytes
+
+        An lz4-compressed pickle byte-string.
+
+    Warning
+    -------
+
+    References between objects are only preserved if they are stored as part of a single object, for
+    example:
+
+    .. code:: python
+
+        # b has a reference to a
+        a = [13]
+        b = {'a': a}
+
+        # references preserved
+        s = dumps((a, b))
+        a_new, b_new = loads(s)
+        b_new['a'].append(7)
+        print(b_new)  # {'a': [13, 7]}
+        print(a_new)  # [13, 7]         <-- updated
+
+        # references not preserved
+        s_a = dumps(a)
+        s_b = dumps(b)
+        a_new = loads(s_a)
+        b_new = loads(s_b)
+        b_new['a'].append(7)
+        print(b_new)  # {'a': [13, 7]}
+        print(a_new)  # [13]            <-- not updated!!
+
+    Therefore, the safest way to create checkpoints is to store the entire state as a single object
+    like a dict or a tuple.
+
+    """
+    return lz4.frame.compress(pickle.dumps(obj))
+
+
+def load(filepath):
+    r"""
+
+    Load an object from a file that was created by :func:`dump(obj, filepath) <dump>`.
+
+    Parameters
+    ----------
+    filepath : str
+
+        File to load.
+
+    """
+    with lz4.frame.open(filepath, 'rb') as f:
+        return pickle.loads(f.read())
+
+
+def loads(s):
+    r"""
+
+    Load an object from a byte-string that was created by :func:`dumps(obj) <dumps>`.
+
+    Parameters
+    ----------
+    s : str
+
+        An lz4-compressed pickle byte-string.
+
+    """
+    return pickle.loads(lz4.frame.decompress(s))
+
+
 def _reload(module, reload_all, reloaded, logger):
     if isinstance(module, ModuleType):
         module_name = module.__name__
@@ -381,9 +527,7 @@ def generate_gif(env, filepath, policy=None, resize_to=None, duration=50, max_ep
     frames.append(frame)
 
     # generate gif
-    dirname = os.path.dirname(filepath)
-    if dirname:
-        os.makedirs(dirname, exist_ok=True)
+    os.makedirs(os.path.dirname(filepath) or '.', exist_ok=True)
     frames[0].save(
         fp=filepath, format='GIF', append_images=frames[1:], save_all=True,
         duration=duration, loop=0)
@@ -564,12 +708,21 @@ def pretty_repr(o, d=0):
 
     """
     i = "  "  # indentation string
-    if isinstance(o, (jnp.ndarray, onp.ndarray)):
+    if isinstance(o, (jnp.ndarray, onp.ndarray, pd.Index)):
         try:
             summary = f", min={onp.min(o):.3g}, median={onp.median(o):.3g}, max={onp.max(o):.3g}"
         except Exception:
             summary = ""
         return f"array(shape={o.shape}, dtype={str(o.dtype)}{summary:s})"
+    if isinstance(o, (pd.Series, pd.DataFrame)):
+        sep = ',\n' + i * (d + 1)
+        items = zip(('index', 'data'), (o.index, o.values))
+        body = sep + sep.join(f"{k}={pretty_repr(v, d + 1)}" for k, v in items)
+        return f"{type(o).__name__}({body})"
+    if isinstance(o, NamedTuple) or hasattr(o, '_asdict'):
+        sep = '\n' + i * (d + 1)
+        body = sep + sep.join(f"{k}={pretty_repr(v, d + 1)}" for k, v in o._asdict().items())
+        return f"{type(o).__name__}({body})"
     if isinstance(o, tuple):
         sep = ',\n' + i * (d + 1)
         body = '\n' + i * (d + 1) + sep.join(f"{pretty_repr(v, d + 1)}" for v in o)
@@ -578,17 +731,30 @@ def pretty_repr(o, d=0):
         sep = ',\n' + i * (d + 1)
         body = '\n' + i * (d + 1) + sep.join(f"{pretty_repr(v, d + 1)}" for v in o)
         return f"[{body}]"
-    if hasattr(o, '_asdict'):
-        sep = '\n' + i * (d + 1)
-        body = sep + sep.join(f"{k}={pretty_repr(v, d + 1)}" for k, v in o._asdict().items())
-        return f"{type(o).__name__}({body})"
+
     if hasattr(o, 'items'):
-        sep = '\n' + i * (d + 1)
-        body = sep + sep.join(f"'{k}': {pretty_repr(v, d + 1)}" for k, v in o.items())
+        sep = ',\n' + i * (d + 1)
+        body = '\n' + i * (d + 1) + sep.join(
+            f"{repr(k)}: {pretty_repr(v, d + 1)}" for k, v in o.items())
         return f"{{{body}}}"
     return repr(o)
 
 
+def pretty_print(obj):
+    r"""
+
+    Print :func:`pretty_repr(obj) <coax.utils.pretty_repr>`.
+
+    Parameters
+    ----------
+    obj : object
+
+        Any object.
+
+    """
+    print(pretty_repr(obj))
+
+
 def getattr_safe(obj, name, default=None):
     """
 
